# -*- coding: utf-8 -*-

"""
Module for defining radiometric indices.
"""

import logging
import pathlib
from abc import ABCMeta, abstractmethod
from matplotlib import cm
import os

# type annotations
from typing import Union, List

from .indices_functions import (create_raw_ndvi, create_raw_ndwi, create_raw_ndwimcf,
                                create_raw_bigr, create_raw_birnir, create_raw_bibg,
                                create_raw_mndwi,
                                create_masked_indice, index_tiff_2_jp2)
from .colormap import matplotlib_colormap_to_rgb, create_colormap, create_rvb

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


class Indice(metaclass=ABCMeta):
    """Indice abstract base class. Every indice class must implement these
    class attributes and object methods.

    * ``_name``: Name of the indice. Must be unique and without spaces.
    * ``_filename_template``: Template of the indice filename output. Will be
        filled with the product's identifier and the file extension
        e.g.: *{product_identifier}_INDICE_NAME{ext}*
    * ``_ext``: Extension of the indice filename. (eg: ".tif", ".jp2")

    process_indice(out_path, nodata_clouds, quicklook):
    """
    @property
    @abstractmethod
    def name(self):
        pass

    @property
    @abstractmethod
    def filename_template(self):
        pass

    @property
    @abstractmethod
    def ext(self):
        pass

    @abstractmethod
    def process_indice(self, out_path, reprocess, nodata_clouds, quicklook):
        pass


class Ndvi(Indice):
    """
    NDVI = (NIR-VIR) / (NIR+VIR)

    NIR: band 08
    VIR: band 04
    """
    name = "NDVI"
    filename_template = "{product_identifier}_NDVI{ext}"
    ext = ".jp2"
    ext_raw = ".tif"
    colormap = cm.RdYlGn
    # _colors_table_path = functions_data_path = Path("../share/data/RdYlGn.lut")

    def __init__(self, l2a_product_object):
        if l2a_product_object is None:
            raise ValueError("A L2aProduct object must be provided")
        else:
            self.l2a_product = l2a_product_object

        # output path
        self.out_path = None

        # filenames
        self.indice_stem = self.filename_template.format(product_identifier=self.l2a_product.identifier, ext="")
        self.indice_filename = self.indice_stem + self.ext
        self.indice_raw = self.indice_stem + self.ext_raw

    def process_indice(
            self,
            out_path: pathlib.PosixPath,
            reprocess: bool = False,
            nodata_clouds: bool = False,
            quicklook: bool = False
    ) -> None:
        """ process

        :param out_path:
        :param reprocess:
        :param nodata_clouds:
        :param quicklook:
        """
        self.out_path = out_path
        if (out_path / self.indice_filename).exists() and not reprocess:
            logger.info("{} already exists".format(self.indice_filename))
        else:
            create_raw_ndvi(nir_path=self.l2a_product.b08_10m,
                            vir_path=self.l2a_product.b04_10m,
                            out_path=(out_path / self.indice_raw))
            index_tiff_2_jp2(img_path=(out_path / self.indice_raw),
                            out_path=(out_path / self.indice_filename))

        if nodata_clouds:
            if self.l2a_product.user_cloud_mask is None:
                raise ValueError("Cloud mask does not exist")
            masked_indice_filename = self.indice_stem + "_MASKED" + self.ext
            masked_indice_raw = self.indice_stem + "_MASKED" + self.ext_raw
            
            if (out_path / masked_indice_filename).exists() and not reprocess:
                logger.info("{} already exists".format(masked_indice_filename))
            else:
                if (out_path / self.indice_raw).exists():
                    ndvi_name = (out_path / self.indice_raw)
                else:
                    ndvi_name = (out_path / self.indice_filename)
                create_masked_indice(indice_path=ndvi_name,
                                     cloud_mask_path=self.l2a_product.user_cloud_mask,
                                     out_path=(out_path / masked_indice_raw))
                index_tiff_2_jp2(img_path=(out_path / masked_indice_raw),
                                out_path=(out_path / masked_indice_filename))
                os.remove(str(out_path / masked_indice_raw))
        
        try:
            os.remove(str(out_path / self.indice_raw))
            logger.info("Removing {}".format(self.indice_raw))
        except:
            pass
        
        if quicklook:
            cmap = matplotlib_colormap_to_rgb(self.colormap, revers=False)

            quicklook_filename = self.indice_stem + "_QUICKLOOK.tif"

            if (self.out_path / quicklook_filename).exists() and not reprocess:
                logger.info("{} already exists".format(quicklook_filename))
            else:
                logger.info("creating quicklook")
#                create_colormap(raster=(self.out_path / self.indice_filename),
#                                cloud_mask=self.l2a_product.user_cloud_mask,
#                                # lut_dict=get_colormap(self._colors_table_path.absolute()),
#                                lut_dict=cmap, clouds_color="white",
#                                out_path=(self.out_path / quicklook_filename))
                #~ create_colormap2(raster=(self.out_path / self.indice_filename),
                                #~ cloud_mask=self.l2a_product.user_cloud_mask,
                                #~ # lut_dict=get_colormap(self._colors_table_path.absolute()),
                                #~ lut_dict=cmap, clouds_color="white",
                                #~ out_path=(self.out_path / quicklook_filename))
                create_rvb(raster=(self.out_path / self.indice_filename),
                                cloud_mask=self.l2a_product.user_cloud_mask,
                                # lut_dict=get_colormap(self._colors_table_path.absolute()),
                                lut_dict=cmap, clouds_color="white",
                                out_path=(self.out_path / quicklook_filename))


class NdwiGao(Indice):
    """
    NDWI(Gao) = (NIR-SWIR) / (NIR+SWIR)

    NIR: band 08
    SWIR: band 11
    """
    name = "NDWIGAO"
    filename_template = "{product_identifier}_NDWIGAO{ext}"
    ext = ".jp2"
    ext_raw = ".tif"
    colormap = cm.RdYlBu

    def __init__(self, l2a_product_object):
        if l2a_product_object is None:
            raise ValueError("A L2aProduct object must be provided")
        else:
            self.l2a_product = l2a_product_object

        # output path
        self.out_path = None

        # filenames
        self.indice_stem = self.filename_template.format(product_identifier=self.l2a_product.identifier, ext="")
        self.indice_filename = self.indice_stem + self.ext
        self.indice_raw = self.indice_stem + self.ext_raw

    def process_indice(
            self,
            out_path: pathlib.PosixPath,
            reprocess: bool = False,
            nodata_clouds: bool = False,
            quicklook: bool = False
    ) -> None:
        """ process  """
        self.out_path = out_path

        if (out_path / self.indice_filename).exists() and not reprocess:
            logger.info("{} already exists".format(self.indice_filename))
        else:
            create_raw_ndwi(nir_path=self.l2a_product.b08_10m,
                            swir_path=self.l2a_product.b11_20m,
                            out_path=(out_path / self.indice_raw))
            index_tiff_2_jp2(img_path=(out_path / self.indice_raw),
                            out_path=(out_path / self.indice_filename))

        if nodata_clouds:
            if self.l2a_product.user_cloud_mask is None:
                raise ValueError("Cloud mask does not exist")
            masked_indice_filename = self.indice_stem + "_MASKED" + self.ext
            masked_indice_raw = self.indice_stem + "_MASKED" + self.ext_raw

            if (out_path / masked_indice_filename).exists() and not reprocess:
                logger.info("{} already exists".format(masked_indice_filename))
            else:
                if (out_path / self.indice_raw).exists():
                    ndwi_name = (out_path / self.indice_raw)
                else:
                    ndwi_name = (out_path / self.indice_filename)
                create_masked_indice(indice_path=ndwi_name,
                                     cloud_mask_path=self.l2a_product.user_cloud_mask,
                                     out_path=(out_path / masked_indice_raw))
                index_tiff_2_jp2(img_path=(out_path / masked_indice_raw),
                                out_path=(out_path / masked_indice_filename))
                os.remove(str(out_path / masked_indice_raw))
        
        try:
            os.remove(str(out_path / self.indice_raw))
            logger.info("Removing {}".format(self.indice_raw))
        except:
            pass
        
        if quicklook:
            cmap = matplotlib_colormap_to_rgb(self.colormap, revers=False)

            quicklook_filename = self.indice_stem + "_QUICKLOOK.tif"
            if (self.out_path / quicklook_filename).exists() and not reprocess:
                logger.info("{} already exists".format(quicklook_filename))
            else:
                logger.info("creating quicklook")
#                create_colormap(raster=(self.out_path / self.indice_filename),
#                                cloud_mask=self.l2a_product.user_cloud_mask,
#                                lut_dict=cmap, clouds_color="white",
#                                out_path=(self.out_path / quicklook_filename))
                create_rvb(raster=(self.out_path / self.indice_filename),
                                cloud_mask=self.l2a_product.user_cloud_mask,
                                lut_dict=cmap, clouds_color="white",
                                out_path=(self.out_path / quicklook_filename))


class NdwiMcf(Indice):
    """
    NDWI(McFeeters) = (GREEN-NIR) / (GREEN+NIR)

    GREEN: band 03
    NIR: band 08
    """
    name = "NDWIMCF"
    filename_template = "{product_identifier}_NDWIMCF{ext}"
    ext = ".jp2"
    ext_raw = ".tif"
    #~ colormap = cm.RdYlBu
    colormap = cm.colors.LinearSegmentedColormap.from_list("", ["green", "white", "blue"])


    def __init__(self, l2a_product_object):
        if l2a_product_object is None:
            raise ValueError("A L2aProduct object must be provided")
        else:
            self.l2a_product = l2a_product_object

        # output path
        self.out_path = None

        # filenames
        self.indice_stem = self.filename_template.format(product_identifier=self.l2a_product.identifier, ext="")
        self.indice_filename = self.indice_stem + self.ext
        self.indice_raw = self.indice_stem + self.ext_raw

    def process_indice(
            self,
            out_path: pathlib.PosixPath,
            reprocess: bool = False,
            nodata_clouds: bool = False,
            quicklook: bool = False
    ) -> None:
        """ process  """
        self.out_path = out_path

        if (out_path / self.indice_filename).exists() and not reprocess:
            logger.info("{} already exists".format(self.indice_filename))
        else:
            create_raw_ndwimcf(nir_path=self.l2a_product.b08_10m,
                            green_path=self.l2a_product.b03_10m,
                            out_path=(out_path / self.indice_raw))
            index_tiff_2_jp2(img_path=(out_path / self.indice_raw),
                            out_path=(out_path / self.indice_filename))

        if nodata_clouds:
            if self.l2a_product.user_cloud_mask is None:
                raise ValueError("Cloud mask does not exist")
            masked_indice_filename = self.indice_stem + "_MASKED" + self.ext
            masked_indice_raw = self.indice_stem + "_MASKED" + self.ext_raw

            if (out_path / masked_indice_filename).exists() and not reprocess:
                logger.info("{} already exists".format(masked_indice_filename))
            else:
                if (out_path / self.indice_raw).exists():
                    ndwimcf_name = (out_path / self.indice_raw)
                else:
                    ndwimcf_name = (out_path / self.indice_filename)
                create_masked_indice(indice_path=ndwimcf_name,
                                     cloud_mask_path=self.l2a_product.user_cloud_mask,
                                     out_path=(out_path / masked_indice_raw))
                index_tiff_2_jp2(img_path=(out_path / masked_indice_raw),
                                out_path=(out_path / masked_indice_filename))
                os.remove(str(out_path / masked_indice_raw))
        
        try:
            os.remove(str(out_path / self.indice_raw))
            logger.info("Removing {}".format(self.indice_raw))
        except:
            pass
        
        if quicklook:
            cmap = matplotlib_colormap_to_rgb(self.colormap, revers=False)

            quicklook_filename = self.indice_stem + "_QUICKLOOK.tif"
            if (self.out_path / quicklook_filename).exists() and not reprocess:
                logger.info("{} already exists".format(quicklook_filename))
            else:
                logger.info("creating quicklook")
#                create_colormap(raster=(self.out_path / self.indice_filename),
#                                cloud_mask=self.l2a_product.user_cloud_mask,
#                                lut_dict=cmap, clouds_color="white",
#                                out_path=(self.out_path / quicklook_filename))
                create_rvb(raster=(self.out_path / self.indice_filename),
                                cloud_mask=self.l2a_product.user_cloud_mask,
                                lut_dict=cmap, clouds_color="white",
                                out_path=(self.out_path / quicklook_filename))


##############################
######## New Indices #########
##############################

class BIGR(Indice):
    """
    Brightness Index Green Red = ( (GREEN² + RED²)/2 ) ^ 0.5

    GREEN: band 03
    RED: band 04
    """
    name = "BIGR"
    filename_template = "{product_identifier}_BIGR{ext}"
    ext = ".jp2"
    ext_raw = ".tif"
    colormap = cm.pink
    #~ colormap = cm.colors.LinearSegmentedColormap.from_list("", ["black", "white"])


    def __init__(self, l2a_product_object):
        if l2a_product_object is None:
            raise ValueError("A L2aProduct object must be provided")
        else:
            self.l2a_product = l2a_product_object

        # output path
        self.out_path = None

        # filenames
        self.indice_stem = self.filename_template.format(product_identifier=self.l2a_product.identifier, ext="")
        self.indice_filename = self.indice_stem + self.ext
        self.indice_raw = self.indice_stem + self.ext_raw

    def process_indice(
            self,
            out_path: pathlib.PosixPath,
            reprocess: bool = False,
            nodata_clouds: bool = False,
            quicklook: bool = False
    ) -> None:
        """ process  """
        self.out_path = out_path

        if (out_path / self.indice_filename).exists() and not reprocess:
            logger.info("{} already exists".format(self.indice_filename))
        else:
            create_raw_bigr(red_path=self.l2a_product.b04_10m,
                            green_path=self.l2a_product.b03_10m,
                            out_path=(out_path / self.indice_raw))
            index_tiff_2_jp2(img_path=(out_path / self.indice_raw),
                            out_path=(out_path / self.indice_filename))

        if nodata_clouds:
            if self.l2a_product.user_cloud_mask is None:
                raise ValueError("Cloud mask does not exist")
            masked_indice_filename = self.indice_stem + "_MASKED" + self.ext
            masked_indice_raw = self.indice_stem + "_MASKED" + self.ext_raw

            if (out_path / masked_indice_filename).exists() and not reprocess:
                logger.info("{} already exists".format(masked_indice_filename))
            else:
                if (out_path / self.indice_raw).exists():
                    bigr_name = (out_path / self.indice_raw)
                else:
                    bigr_name = (out_path / self.indice_filename)
                create_masked_indice(indice_path=bigr_name,
                                     cloud_mask_path=self.l2a_product.user_cloud_mask,
                                     out_path=(out_path / masked_indice_raw))
                index_tiff_2_jp2(img_path=(out_path / masked_indice_raw),
                                out_path=(out_path / masked_indice_filename))
                os.remove(str(out_path / masked_indice_raw))
        
        try:
            os.remove(str(out_path / self.indice_raw))
            logger.info("Removing {}".format(self.indice_raw))
        except:
            pass
        
        if quicklook:
            cmap = matplotlib_colormap_to_rgb(self.colormap, revers=False)

            quicklook_filename = self.indice_stem + "_QUICKLOOK.tif"
            if (self.out_path / quicklook_filename).exists() and not reprocess:
                logger.info("{} already exists".format(quicklook_filename))
            else:
                logger.info("creating quicklook")
                create_rvb(raster=(self.out_path / self.indice_filename),
                                cloud_mask=self.l2a_product.user_cloud_mask,
                                lut_dict=cmap, clouds_color="white",
                                out_path=(self.out_path / quicklook_filename),
                                stretch=(0,2500))


class BIRNIR(Indice):
    """
    Brightness Index Red Near InfraRed = ( (RED² + NIR²)/2 ) ^ 0.5

    NIR: band 08
    RED: band 04
    """
    name = "BIRNIR"
    filename_template = "{product_identifier}_BIRNIR{ext}"
    ext = ".jp2"
    ext_raw = ".tif"
    colormap = cm.afmhot
    #~ colormap = cm.colors.LinearSegmentedColormap.from_list("", ["black", "white"])


    def __init__(self, l2a_product_object):
        if l2a_product_object is None:
            raise ValueError("A L2aProduct object must be provided")
        else:
            self.l2a_product = l2a_product_object

        # output path
        self.out_path = None

        # filenames
        self.indice_stem = self.filename_template.format(product_identifier=self.l2a_product.identifier, ext="")
        self.indice_filename = self.indice_stem + self.ext
        self.indice_raw = self.indice_stem + self.ext_raw

    def process_indice(
            self,
            out_path: pathlib.PosixPath,
            reprocess: bool = False,
            nodata_clouds: bool = False,
            quicklook: bool = False
    ) -> None:
        """ process  """
        self.out_path = out_path

        if (out_path / self.indice_filename).exists() and not reprocess:
            logger.info("{} already exists".format(self.indice_filename))
        else:
            create_raw_birnir(red_path=self.l2a_product.b04_10m,
                            nir_path=self.l2a_product.b08_10m,
                            out_path=(out_path / self.indice_raw))
            index_tiff_2_jp2(img_path=(out_path / self.indice_raw),
                            out_path=(out_path / self.indice_filename))

        if nodata_clouds:
            if self.l2a_product.user_cloud_mask is None:
                raise ValueError("Cloud mask does not exist")
            masked_indice_filename = self.indice_stem + "_MASKED" + self.ext
            masked_indice_raw = self.indice_stem + "_MASKED" + self.ext_raw

            if (out_path / masked_indice_filename).exists() and not reprocess:
                logger.info("{} already exists".format(masked_indice_filename))
            else:
                if (out_path / self.indice_raw).exists():
                    birnir_name = (out_path / self.indice_raw)
                else:
                    birnir_name = (out_path / self.indice_filename)
                create_masked_indice(indice_path=birnir_name,
                                     cloud_mask_path=self.l2a_product.user_cloud_mask,
                                     out_path=(out_path / masked_indice_raw))
                index_tiff_2_jp2(img_path=(out_path / masked_indice_raw),
                                out_path=(out_path / masked_indice_filename))
                os.remove(str(out_path / masked_indice_raw))
        
        try:
            os.remove(str(out_path / self.indice_raw))
            logger.info("Removing {}".format(self.indice_raw))
        except:
            pass
        
        if quicklook:
            cmap = matplotlib_colormap_to_rgb(self.colormap, revers=False)

            quicklook_filename = self.indice_stem + "_QUICKLOOK.tif"
            if (self.out_path / quicklook_filename).exists() and not reprocess:
                logger.info("{} already exists".format(quicklook_filename))
            else:
                logger.info("creating quicklook")
                create_rvb(raster=(self.out_path / self.indice_filename),
                                cloud_mask=self.l2a_product.user_cloud_mask,
                                lut_dict=cmap, clouds_color="white",
                                out_path=(self.out_path / quicklook_filename),
                                stretch=(0,5000))


class BIBG(Indice):
    """
    Brightness Index Blue Green = ( (BLUE² + GREEN²)/2 ) ^ 0.5

    BLUE: band 02
    GREEN: band 03
    """
    name = "BIBG"
    filename_template = "{product_identifier}_BIBG{ext}"
    ext = ".jp2"
    ext_raw = ".tif"
    colormap = cm.bone
    #~ colormap = cm.colors.LinearSegmentedColormap.from_list("", ["black", "white"])


    def __init__(self, l2a_product_object):
        if l2a_product_object is None:
            raise ValueError("A L2aProduct object must be provided")
        else:
            self.l2a_product = l2a_product_object

        # output path
        self.out_path = None

        # filenames
        self.indice_stem = self.filename_template.format(product_identifier=self.l2a_product.identifier, ext="")
        self.indice_filename = self.indice_stem + self.ext
        self.indice_raw = self.indice_stem + self.ext_raw

    def process_indice(
            self,
            out_path: pathlib.PosixPath,
            reprocess: bool = False,
            nodata_clouds: bool = False,
            quicklook: bool = False
    ) -> None:
        """ process  """
        self.out_path = out_path

        if (out_path / self.indice_filename).exists() and not reprocess:
            logger.info("{} already exists".format(self.indice_filename))
        else:
            create_raw_bibg(blue_path=self.l2a_product.b02_10m,
                            green_path=self.l2a_product.b03_10m,
                            out_path=(out_path / self.indice_raw))
            index_tiff_2_jp2(img_path=(out_path / self.indice_raw),
                            out_path=(out_path / self.indice_filename))

        if nodata_clouds:
            if self.l2a_product.user_cloud_mask is None:
                raise ValueError("Cloud mask does not exist")
            masked_indice_filename = self.indice_stem + "_MASKED" + self.ext
            masked_indice_raw = self.indice_stem + "_MASKED" + self.ext_raw

            if (out_path / masked_indice_filename).exists() and not reprocess:
                logger.info("{} already exists".format(masked_indice_filename))
            else:
                if (out_path / self.indice_raw).exists():
                    bibg_name = (out_path / self.indice_raw)
                else:
                    bibg_name = (out_path / self.indice_filename)
                create_masked_indice(indice_path=bibg_name,
                                     cloud_mask_path=self.l2a_product.user_cloud_mask,
                                     out_path=(out_path / masked_indice_raw))
                index_tiff_2_jp2(img_path=(out_path / masked_indice_raw),
                                out_path=(out_path / masked_indice_filename))
                os.remove(str(out_path / masked_indice_raw))
        
        try:
            os.remove(str(out_path / self.indice_raw))
            logger.info("Removing {}".format(self.indice_raw))
        except:
            pass
        
        if quicklook:
            cmap = matplotlib_colormap_to_rgb(self.colormap, revers=False)

            quicklook_filename = self.indice_stem + "_QUICKLOOK.tif"
            if (self.out_path / quicklook_filename).exists() and not reprocess:
                logger.info("{} already exists".format(quicklook_filename))
            else:
                logger.info("creating quicklook")
                create_rvb(raster=(self.out_path / self.indice_filename),
                                cloud_mask=self.l2a_product.user_cloud_mask,
                                lut_dict=cmap, clouds_color="white",
                                out_path=(self.out_path / quicklook_filename),
                                stretch=(0,2500))


class Mndwi(Indice):
    """
    MNDWI = (GREEN-SWIR) / (GREEN+SWIR)

    GREEN: band 03
    SWIR: band 11
    """
    name = "MNDWI"
    filename_template = "{product_identifier}_MNDWI{ext}"
    ext = ".jp2"
    ext_raw = ".tif"
    colormap = cm.BrBG

    def __init__(self, l2a_product_object):
        if l2a_product_object is None:
            raise ValueError("A L2aProduct object must be provided")
        else:
            self.l2a_product = l2a_product_object

        # output path
        self.out_path = None

        # filenames
        self.indice_stem = self.filename_template.format(product_identifier=self.l2a_product.identifier, ext="")
        self.indice_filename = self.indice_stem + self.ext
        self.indice_raw = self.indice_stem + self.ext_raw

    def process_indice(
            self,
            out_path: pathlib.PosixPath,
            reprocess: bool = False,
            nodata_clouds: bool = False,
            quicklook: bool = False
    ) -> None:
        """ process  """
        self.out_path = out_path

        if (out_path / self.indice_filename).exists() and not reprocess:
            logger.info("{} already exists".format(self.indice_filename))
        else:
            create_raw_mndwi(green_path=self.l2a_product.b03_10m,
                            swir_path=self.l2a_product.b11_20m,
                            out_path=(out_path / self.indice_raw))
            index_tiff_2_jp2(img_path=(out_path / self.indice_raw),
                            out_path=(out_path / self.indice_filename))

        if nodata_clouds:
            if self.l2a_product.user_cloud_mask is None:
                raise ValueError("Cloud mask does not exist")
            masked_indice_filename = self.indice_stem + "_MASKED" + self.ext
            masked_indice_raw = self.indice_stem + "_MASKED" + self.ext_raw

            if (out_path / masked_indice_filename).exists() and not reprocess:
                logger.info("{} already exists".format(masked_indice_filename))
            else:
                if (out_path / self.indice_raw).exists():
                    mndwi_name = (out_path / self.indice_raw)
                else:
                    mndwi_name = (out_path / self.indice_filename)
                create_masked_indice(indice_path=mndwi_name,
                                     cloud_mask_path=self.l2a_product.user_cloud_mask,
                                     out_path=(out_path / masked_indice_raw))
                index_tiff_2_jp2(img_path=(out_path / masked_indice_raw),
                                out_path=(out_path / masked_indice_filename))
                os.remove(str(out_path / masked_indice_raw))
        
        try:
            os.remove(str(out_path / self.indice_raw))
            logger.info("Removing {}".format(self.indice_raw))
        except:
            pass
        
        if quicklook:
            cmap = matplotlib_colormap_to_rgb(self.colormap, revers=False)

            quicklook_filename = self.indice_stem + "_QUICKLOOK.tif"
            if (self.out_path / quicklook_filename).exists() and not reprocess:
                logger.info("{} already exists".format(quicklook_filename))
            else:
                logger.info("creating quicklook")
                create_rvb(raster=(self.out_path / self.indice_filename),
                                cloud_mask=self.l2a_product.user_cloud_mask,
                                lut_dict=cmap, clouds_color="white",
                                out_path=(self.out_path / quicklook_filename))


class IndicesCollectionMeta(type):
    """Adds special methods to IndicesCollection class.

    """
    def __iter__(cls):
        return iter(cls._indices_classes)

    def __contains__(cls, indice):
        return indice in cls._indices_classes

    def __len__(cls):
        return len(cls._indices_classes)

    def __str__(cls):
        return str(cls.list)

    @property
    def available_indices(cls) -> List[str]:
        """Returns indices classes names."""
        return list(cls._indices_classes.keys())

    @property
    def list(cls) -> List[str]:
        """Returns indices classes names."""
        return list(cls._indices_classes.keys())

    def get_indice_cls(cls, indice: str) -> Union["Indice", None]:
        """
        Returns the indice class.

        :param indice: name of the indice.
        """
        try:
            indice_cls = cls._indices_classes[indice]
        except KeyError:
            logger.error("Indice: {} is not defined".format(indice))
            return None
        else:
            return indice_cls


class IndicesCollection(metaclass=IndicesCollectionMeta):
    """Lists defined indices classes.

    Usage:
        >>> IndicesCollection.list

    """
    # TODO: Implement class as a singleton.

    _indices_classes = {cls.__dict__["name"]: cls for cls in Indice.__subclasses__()}
