# -*- coding: utf-8 -*-

"""
Module for managing products and tiles in the library and temp folders.
"""

import os
import shutil
import logging
import pathlib

from pathlib import Path
# type annotations
from typing import List, Dict

from .config import Config, SHARED_DATA
from .products import L1cProduct
from .tiles import Tile
from .utils import human_size, human_size_decimal

s2_tiles_index = SHARED_DATA.get("tiles_index")

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class Library:
    """Class for listing L1C, L2A and indices tiles in the library folders.

    """
    _temp_path = Path(Config().get("temp_path"))
    _l1c_path = Path(Config().get("l1c_path"))
    _l2a_path = Path(Config().get("l2a_path"))
    _indices_path = Path(Config().get("indices_path"))

    def __init__(self):
        self._l1c = [f.name for f in self._l1c_path.glob("*")]
        self._l2a = [f.name for f in self._l2a_path.glob("*")]
        self._indices = dict()

        for indice in [f.name for f in self._indices_path.glob("*")]:
            self.__dict__[indice.lower()] = [f.name for f in self._indices_path.glob("{}/*".format(indice))]

    @property
    def l1c(self) -> List[str]:
        """Returns tiles in the L1C library folder."""
        return self._l1c

    @property
    def l2a(self) -> List[str]:
        """Returns tiles in the L2A library folder."""
        return self._l2a

    @property
    def indices(self) -> Dict[str, pathlib.PosixPath]:
        """Returns tiles in the indices library folder."""
        return self._indices
        
    def clean(self, 
              clean_list: list = [],
              remove_indice_tif: bool = False, 
              remove: bool = False):
        """Function to clean corrupted files during processing errors from whole Library
        or selected Tiles. Calls the clean_lib function from Tile class.
        
        :param clean_list: list of tiles to be cleaned. If not provided, will process all l1c tiles.
        :param remove_indice_tif: bool. If True will remove present TIFF files present in indice folders. Default value False.
        :param remove: bool. If True will effectively remove corrupted files, if False will just list identified problems. Default value False.
        
        """
        nb_id = 0
        nb_rm = 0
        if not clean_list:
            clean_list = self.l1c
        for t in clean_list:
            try:
                til = Tile(t)
                counts = til.clean_lib(remove=remove, remove_indice_tif=remove_indice_tif)
                nb_id += counts["identified_problems"]
                nb_rm += counts["removed_problems"]
                logger.info("{} - {}".format(til, counts))
            except:
                pass
        return {"identified_problems": nb_id, "removed_problems": nb_rm}
            
    def archive_l1c(self,
                    archive_list: list = [],
                    size_only: bool = False,
                    ):
        """
        Function to archive l1c products from library folder to l1c_archive_path.
        Calls the archive_l1c function from Tile class, see there for details.
        
        :param archive_list: list of tiles to archive. If not provided, will process all l1c tiles.
        :param size_only: if True, only gives sizes
        """        
        
        total_size = 0 
        if not archive_list:
            archive_list = self.l1c
        for t in archive_list:
            try:
                logger.info(t)
                til = Tile(t)
                size_tile = til.archive_l1c(size_only = size_only)
                if size_only:
                    total_size += size_tile
            except:
                pass
        logger.info("Total l1c size to move: {}".format(human_size_decimal(total_size)))
        
    def update_latest_ql(self):
        """
        Produce or update the latest quicklook for the L2A library tiles
        """
        for tile in self.l2a:
            Tile(tile).update_latest_ql()
    
    def archive_l2a(self,
                    archive_list: list = [],
                    size_only: bool = False,
                    ):
                        
        """
        Function to archive l2a products from library folder to l2a_archive_path.
        Calls the archive_l2a function from Tile class, see there for details.
        
        :param archive_list: list of tiles to archive. If not provided, will process all l2a tiles.
        :param size_only: if True, only gives sizes
        """        
        
        total_size = 0 
        if not archive_list:
            archive_list = self.l2a
        for t in archive_list:
            try:
                logger.info(t)
                til = Tile(t)
                size_tile = til.archive_l2a(size_only = size_only)
                if size_only:
                    total_size += size_tile
            except:
                pass
        logger.info("Total l2a size to move: {}".format(human_size_decimal(total_size)))
        
    def archive_all(self,
                    archive_list: list = [],
                    size_only: bool = False,
                    ):
        total_size = 0 
        if not archive_list:
            archive_list = self.l1c
        for t in archive_list:
            try:
                logger.info(t)
                til = Tile(t)
                size_tile = til.archive_all(size_only = size_only)
                total_size += size_tile
            except:
                pass
        logger.info("Total size to move: {}".format(human_size_decimal(total_size)))
        
    def update_latest_ql(self):
        """
        Produce or update the latest quicklook for the L2A library tiles
        """
        for tile in self.l2a:
            Tile(tile).update_latest_ql()

class TempContainer:
    """Class for managing a downloaded L1C products.

    :param l1c_identifier: L1C product's identifier.
    :param tile: product's tile name.
    """
    _temp_path = Path(Config().get("temp_path"))

    def __init__(self, l1c_identifier=None, tile: str = None):
        self.tile = tile
        self.temp_id = l1c_identifier + "_TEMP"
        self.temp_path = self._temp_path / self.temp_id
        self.l1c = L1cProduct(identifier=l1c_identifier, tile=tile, path=self.temp_path)

    def create_temp_folder(self) -> "TempContainer":
        """Creates the L1C product's temporary folder in the TEMP library folder."""
        logger.info("{}: creating temp folder".format(self.l1c.identifier))
        try:
            self.temp_path.mkdir(exist_ok=True)
        except FileNotFoundError:
            print("Invalid temp folder path:", self.temp_path)
            raise
        return self

    def unzip_l1c(self) -> "TempContainer":
        """Unzips the downloaded L1C zip file in the product's temporary folder."""
        zip_file = self.temp_path / (self.l1c.identifier + ".zip")
        if zip_file.exists():
            logger.info("{} : Unzipping L1C archive".format(self.l1c.identifier))
            shutil.unpack_archive(str(zip_file), str(self.temp_path))
            sorted(zip_file.parent.glob("*.SAFE"))[0].rename(zip_file.parent / (zip_file.stem + ".SAFE"))
            os.remove(str(zip_file))
        return self

    def delete_temp_folder(self) -> "TempContainer":
        """Deletes the L1C product's temporary folder."""
        logger.info("Deleting temp folder: {}".format(self.temp_path))
        if not list(self.temp_path.glob("*.SAFE")):
            try:
                shutil.rmtree(str(self.temp_path))
            except FileNotFoundError:
                logger.warning("Temp folder does not exist: {}".format(self.temp_path))
        else:
            logger.error("Temp folder contains a SAFE product.")
        return self
