# -*- coding: utf-8 -*-

"""
Module for managing products and tiles in the library and temp folders.
"""
import pathlib
import logging
import re
import fiona
import shutil
from PIL import Image
import distutils

from pathlib import Path
from collections import namedtuple
from datetime import datetime
from pprint import pformat
# type annotations
from typing import List, Dict, Iterable
from itertools import chain

from .config import Config, SHARED_DATA
from .utils import str_to_datetime, human_size, getFolderSize
from .indices import IndicesCollection
from .products import L1cProduct, L2aProduct
from .multi_processing import l2a_multiprocessing, cld_multiprocessing, idx_multiprocessing

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


# namedtuple
TileProduct = namedtuple("TileProduct", ["identifier", "date", "cloud_cover"])


class ProductsList:
    """Class for managing tiles products lists. Each item is a TileProduct
    namedtuple.

    """
    def __init__(self):
        self._dict = dict()
        self._time_index = dict()

    @property
    def products(self) -> List[str]:
        """Returns products names."""
        return list(self._dict.keys())

    @property
    def dates(self) -> List[datetime]:
        """Returns products dates."""
        return list(self._time_index.keys())

    @property
    def first(self) -> TileProduct:
        """Returns oldest product."""
        try:
            min_date = min(self.dates)
        except ValueError:
            return TileProduct(None, None, None)
        prod = self._time_index[min_date]
        return TileProduct(prod, min_date, self._dict[prod]["cloud_cover"])

    @property
    def last(self) -> TileProduct:
        """Returns newest product."""
        try:
            max_date = max(self.dates)
        except ValueError:
            return TileProduct(None, None, None)
        prod = self._time_index[max_date]
        return TileProduct(prod, max_date, self._dict[prod]["cloud_cover"])

    def filter_dates(
            self,
            date_min: str = None, date_max: str = None
    ) -> "ProductsList":
        """Filters products list in a time range.

        :param date_min: oldest date.
        :param date_max: newest date.
        """
        min_date = str_to_datetime(date_min, "ymd") if date_min else self.first.date
        max_date = str_to_datetime(date_max, "ymd") if date_max else self.last.date
        filtered = ProductsList()
        for k, v in self._dict.items():
            if min_date.date() <= v["date"].date() <= max_date.date():
                filtered[k] = {"date": v["date"], "cloud_cover": v["cloud_cover"]}
        return filtered

    def filter_clouds(
            self,
            cover_min: int = 0, cover_max: int = 100
    ) -> "ProductsList":
        """Filters products list.
        :param cover_min: minimum cloud coverage.
        :param cover_max: maximum cloud coverage.
        """
        filtered = ProductsList()
        for k, v in self._dict.items():
            if cover_min <= v["cloud_cover"] <= cover_max:
                filtered[k] = {"date": v["date"], "cloud_cover": v["cloud_cover"]}
        return filtered

    def __len__(self) -> int:
        return len(self._dict)

    def __contains__(self, item) -> bool:
        return item in self._dict

    def __repr__(self) -> str:
        return pformat([self[k] for k in self.products], indent=1, width=1)

    def __iter__(self) -> Iterable:
        return (self[k] for k in self._dict)

    def __getitem__(self, item) -> TileProduct:
        if item in self._dict:
            return TileProduct(item, self._dict[item]["date"], self._dict[item]["cloud_cover"])
        else:
            raise KeyError("{} not found".format(item))

    def __setitem__(self, item, value) -> None:
        self._dict[item] = {"date": value["date"], "cloud_cover": value["cloud_cover"]}
        self._time_index[value["date"]] = item


class IndicesList(ProductsList):
    """Class for managing indices products lists.

    """
    @property
    def raws(self) -> "ProductsList":
        filtered = ProductsList()
        for k, v in self._dict.items():
            if not("MASK" in k) and not("QUICKLOOK" in k):
                filtered[k] = {"date": v["date"], "cloud_cover": v["cloud_cover"]}
        return filtered

    @property
    def masks(self) -> "ProductsList":
        filtre = ProductsList()
        for k, v in self._dict.items():
            if "MASK" in k:
                filtre[k] = {"date": v["date"], "cloud_cover": v["cloud_cover"]}
        return filtre

    @property
    def quicklooks(self) -> "ProductsList":
        filtered = ProductsList()
        for k, v in self._dict.items():
            if "QUICKLOOK" in k:
                filtered[k] = {"date": v["date"], "cloud_cover": v["cloud_cover"]}
        return filtered


class Tile:
    """Class for managing tiles in the library.

    :param name: tile's name.

    Usage:
        >>> Tile("40KCC")
    """
    def __init__(self, name: str):
        self.name = name
        self._paths = {"l1c": Path(Config().get("l1c_path")) / name,
                       "l2a": Path(Config().get("l2a_path")) / name,
                       "indices": {}
                       }
        self._indices_path = Path(Config().get("indices_path"))

        self._products = {"l1c": ProductsList(),
                          "l2a": ProductsList(),
                          "cloudmasks" : ProductsList(),
                          "indices": dict()}

        self._get_indices_paths()
        self._get_l1c_list()
        self._get_l2a_list()
        self._get_cloudmasks()
        self._get_indices_list()

    def _get_indices_paths(self) -> None:
        """Updates self._paths"""
        for indice in self._indices_path.glob("*/{}".format(self.name)):
            self._paths["indices"][str(indice.parent.name.lower())] = indice

    def _get_l1c_list(self) -> None:
        """Scans L1C folder and adds products in a ProductsList."""
        for f in self._paths["l1c"].glob("*L1C_*.SAFE"):
            cloud_cover = L1cProduct(identifier=f.name, tile=self.name, path=None).cloud_coverage_assessment
            self._products["l1c"][f.name] = {"date": Tile._get_date(f.name), "cloud_cover": float(cloud_cover)}

    def _get_l2a_list(self) -> None:
        """Scans L2A folder and adds products in a ProductsList."""
        for f in self._paths["l2a"].glob("*L2A*.SAFE"):
            l1c_name = f.name.replace("L2A_", "L1C_").replace("_USER_", "_OPER_")
            try:
                date = self._products["l1c"][l1c_name].date
                cloud_cover = self._products["l1c"][l1c_name].cloud_cover
            except KeyError:
                date = Tile._get_date(f.name)
                cloud_cover = L2aProduct(f.name, self.name).cloud_coverage_assessment
            self._products["l2a"][f.name] = {"date": date, "cloud_cover": float(cloud_cover)}

    def _get_cloudmasks(self) -> None:
        """Scans L2A folder for cloud masks and adds corresponding L2A products in a ProductsList."""
        for f in self._paths["l2a"].glob("*L2A*_CLOUD_MASK.jp2"):
            l1c_name = f.name.replace("L2A_", "L1C_").replace("_USER_", "_OPER_").replace("_CLOUD_MASK.jp2", ".SAFE")
            try:
                date = self._products["l1c"][l1c_name].date
                cloud_cover = self._products["l1c"][l1c_name].cloud_cover
            except KeyError:
                date = Tile._get_date(f.name.replace("_CLOUD_MASK.jp2", ".SAFE"))
                cloud_cover = L2aProduct(f.name.replace("_CLOUD_MASK.jp2", ".SAFE"), self.name).cloud_coverage_assessment
            self._products["cloudmasks"][f.name.replace("_CLOUD_MASK.jp2", ".SAFE")] = {"date": date, "cloud_cover": float(cloud_cover)}

    def _get_indices_list(self) -> None:
        """Scans indices folders and adds products in a IndicesList."""
        for indice, path in self._paths["indices"].items():
            if path.is_dir():
                self._products["indices"][indice] = IndicesList()
                indice_template = IndicesCollection.get_indice_cls(indice.upper()).filename_template
                indice_ext = IndicesCollection.get_indice_cls(indice.upper()).ext
                file_patterns = [indice_ext, 'QUICKLOOK.tif']
                files_selected = []
                for p in file_patterns:
                    files_selected.extend(path.glob("*/*{}".format(p)))
                for f in files_selected:
                    try:
                        indice_pattern = re.sub("{.*?}", "", indice_template)
                        remove_pattern = "{}.*".format(indice_pattern)
                        l2a_name = re.sub(remove_pattern, '', f.name) + ".SAFE"
                        date = self._products["l2a"][l2a_name].date
                        cloud_cover = self._products["l2a"][l2a_name].cloud_cover
                    except KeyError:
                        date = Tile._get_date(f.name)
                        cloud_cover = None
                    self._products["indices"][indice][f.name] = {"date": date, "cloud_cover": cloud_cover}
                self.__dict__[indice] = self._products["indices"][indice]

    @staticmethod
    def is_tile_name_valid(tile_name: str) -> bool:
        """Checks if the tile_name is a valid Sentinel-2 tile.

        :param tile_name: tile's name.
        """
        if not re.match(r"^[0-9]{2}[A-Z]{3}$", tile_name):
            return False
        with fiona.open(str(SHARED_DATA["tiles_index"]), 'r') as tiles_index:
            tiles_names = [tile["properties"]["Name"] for tile in tiles_index]
            return tile_name in tiles_names

    @staticmethod
    def _get_date(product_name: str) -> datetime:
        """Extracts the acquisition date of a products filename.
        """
        date_pattern = re.findall(r"[0-9]{8}T[0-9]{6}", product_name)
        if product_name.startswith("S2A_OPER"):
            date = date_pattern[1]
        else:
            date = date_pattern[0]
        return str_to_datetime(date, "filename")

    def __repr__(self) -> str:
        return "Tile('{}')".format(self.name)

    def __str__(self) -> str:
        return self.name

    @property
    def paths(self) -> Dict[str, pathlib.PosixPath]:
        """Returns all the paths related to a Tile object."""
        return self._paths

    @property
    def l1c(self) -> "ProductsList":
        """Returns tile's L1C products as a ProductsList."""
        return self._products["l1c"]

    @property
    def l2a(self) -> "ProductsList":
        """Returns tile's L2A products as a ProductsList."""
        return self._products["l2a"]

    @property
    def cloudmasks(self) -> "ProductsList":
        """Returns tile's cloud masks products as a ProductsList."""
        return self._products["cloudmasks"]

    @property
    def l1c_missings(self) -> "ProductsList":
        """Returns tile's L2A products that don't have a L1C as a ProductsList."""
        prods_list = ProductsList()
        missings_l1c_set = set(self.l2a.products) - {identifier.replace("L1C_", "L2A_").replace("__OPER__", "_USER_")
                                                     for identifier in self.l1c.products}
        for prod in missings_l1c_set:
            prods_list[prod] = {"date": self._products["l2a"][prod].date,
                                "cloud_cover": self._products["l2a"][prod].cloud_cover}
        return prods_list

    @property
    def l2a_missings(self) -> "ProductsList":
        """Returns tile's L1C products that don't have a L2A as a ProductsList."""
        prods_list = ProductsList()
        missings_l2a_set = set(self.l1c.products) - {identifier.replace("L2A_", "L1C_").replace("_USER_", "_OPER_")
                                                     for identifier in self.l2a.products}
        for prod in missings_l2a_set:
            prods_list[prod] = {"date": self._products["l1c"][prod].date,
                                "cloud_cover": self._products["l1c"][prod].cloud_cover}
        return prods_list

    @property
    def cloudmasks_missings(self) -> "ProductsList":
        """Returns tile's L2A products that don't have a cloud mask as a ProductsList."""
        prods_list = ProductsList()
        missings_l2a_set = set(self.l2a.products) - {identifier for identifier in self.cloudmasks.products}
        for prod in missings_l2a_set:
            prods_list[prod] = {"date": self._products["l2a"][prod].date,
                                "cloud_cover": self._products["l2a"][prod].cloud_cover}
        return prods_list
    
    @property
    def info(self):
        logger.info("l1c: {}".format(len(self.l1c)))
        logger.info("l2a: {}".format(len(self.l2a)))
        logger.info("cloud_masks: {}".format(len(self.cloudmasks)))
        for indice, path in self._paths["indices"].items():
            logger.info("{} (raw / masked): {} / {}".format(indice, len(getattr(self, indice).raws), len(getattr(self, indice).masks)))
    
    @property
    def size(self):
        #~ logger.info("Product: local / archived / total".format(human_size(getFolderSize(str(self.paths['l1c'])))))
        local = getFolderSize(str(self.paths["l1c"]))
        total = getFolderSize(str(self.paths["l1c"]), True)
        logger.info("l1c: {} (local: {} / archived: {})".format(human_size(total),
                                                   human_size(local),
                                                   human_size(total-local),
                                                   ))
        
        #~ logger.info("l2a: {}".format(human_size(getFolderSize(str(self.paths["l2a"]), True))))
        local = getFolderSize(str(self.paths["l2a"]))
        total = getFolderSize(str(self.paths["l2a"]), True)
        logger.info("l2a: {} (local: {} / archived: {})".format(human_size(total),
                                                   human_size(local),
                                                   human_size(total-local),
                                                   ))
        
        for indice, path in self._paths["indices"].items():
            logger.info("{}: {}".format(indice, human_size(getFolderSize(str(path), True))))
            
    def missing_indices(self, indice: str) -> "ProductsList":
        """Returns tile's L2A products that don't have indices as a ProductsList."""
        prods_list = ProductsList()
        try:
            missings_indice_set = set(self.l2a.products) - {identifier.replace("_" + indice.upper() + "_MASKED.jp2", ".SAFE") \
                                                            for identifier in getattr(getattr(self, indice.lower()), 'masks').products}
        except:
            missings_indice_set = set(self.l2a.products)
        for prod in missings_indice_set:
                prods_list[prod] = {"date": self._products["l2a"][prod].date,
                                "cloud_cover": self._products["l2a"][prod].cloud_cover}            
        return prods_list
        
    def compute_l2a(self,
                    date_min: str = None,
                    date_max: str = None,
                    nb_proc: int = 4):
        """
        Compute all missing l2a for l1c products
        """
        l1c_process_list =  []
        l1c_process_list.append(list(p.identifier for p in self.l2a_missings.filter_dates(date_min = date_min, date_max = date_max)))
        l1c_process_list = list(chain.from_iterable(l1c_process_list))
        if l1c_process_list:
            logger.info("{} l1c products to process:".format(len(l1c_process_list)))
            logger.info("{}".format(l1c_process_list))
        else:
            logger.info("All l2a products already computed")
        l2a_res = False
        if l1c_process_list:
            l2a_res = l2a_multiprocessing(l1c_process_list, nb_proc=nb_proc)
        
    def compute_cloudmasks(self,
                            date_min: str = None,
                            date_max: str = None,
                            nb_proc: int = 4):
        """
        Compute all missing cloud masks for l2a products
        """
        
        cld_l2a_process_list =  []
        cld_l2a_process_list.append(list(p.identifier for p in self.cloudmasks_missings.filter_dates(date_min = date_min, date_max = date_max)))
        cld_l2a_process_list = list(chain.from_iterable(cld_l2a_process_list))
        if cld_l2a_process_list:
            logger.info("{} l2a products to process:".format(len(cld_l2a_process_list)))
            logger.info("{}".format(cld_l2a_process_list))
        else:
            logger.info("All cloud masks already computed")
        cld_res = False
        if cld_l2a_process_list:
            cld_res = cld_multiprocessing(cld_l2a_process_list, nb_proc=nb_proc)
    
    def compute_indices(self,
                        indices: list = [],
                        date_min: str = None,
                        date_max: str = None,
                        nb_proc: int = 4):
        """
        Compute all missing indices for l2a products
        - indices are given as a list
        - if indices not provided, will compute missing dates of already existing indices for this tile (no new indice computed)
        - indices won't be masked if no cloud masks are present, you have to compute cloudmasks first
        """
        if not indices:
            indices = list(self._paths["indices"].keys())
        #~ else:
        indices = [indice.upper() for indice in indices]
        indices_l2a_process_list =  []
        for i in indices:
            l2a_list = [p.identifier for p in self.missing_indices(i).filter_dates(date_min = date_min, date_max = date_max)]
            for j in l2a_list:
                indices_l2a_process_list.append([j, i])
        if indices_l2a_process_list:
            logger.info("{} indice products to process:".format(len(indices_l2a_process_list)))
            logger.info("{}".format(indices_l2a_process_list))
        else:
            logger.info("All indices already computed")
        indices_res = False
        if indices_l2a_process_list:
            indices_res = idx_multiprocessing(indices_l2a_process_list, nb_proc=nb_proc)

    def clean_lib(self,
                  remove_indice_tif: bool = False,
                  remove: bool = False):
        """
        Function to search and clean corrupted files during processing errors from Tile
        
        - unmoved error l2a products from l1c folder
        - moved error l2a products from l2a folder
        - cloud masks error (0kb)
        - indices error (0kb)        
        
        :param clean_list: list of tiles to be cleaned. If not provided, will process all l1c Tiles.
        :param remove_indice_tif: bool. If True will remove present TIFF files present in indice folders. Default value False.
        :param remove: bool. If True will effectively remove corrupted files, if False will just list identified problems. Default value False.
        
        """
        #~ logger.info("Cleaning {} library".format(self.name))
        
        # identify corrupted jp2 in l1c folder
        nb_id = 0
        nb_rm = 0
        erase_set = set()
        for f in self._paths["l1c"].glob("*/GRANULE/*/IMG_DATA/*.jp2"):
            if f.stat().st_size == 0:
                logger.info("Identified 0b corrupted {} in L1C folder".format(f.name))
                nb_id += 1
                if remove:
                    erase_set.update({f.parent.parent.parent.parent})
        for e in erase_set:
            try:
                logger.info("Removing {} from L1C folder".format(e))
                shutil.rmtree(str(e))
                nb_rm += 1
            except:
                logger.error("Can't remove {} from L1C folder".format(e))
        # identify residual l2a from l1c folder
        for f in self._paths["l1c"].glob("*L2A*.SAFE"):
            logger.info("Identified {} in L1C folder".format(f.name))
            nb_id += 1
            if remove:
                try:
                    shutil.rmtree(str(f))
                    logger.info("Removing {} from L1C folder".format(f.name))
                    nb_rm += 1
                except:
                    logger.error("Can't remove {} from L1C folder".format(f.name))
        # identify missing jp2 in L2A folder
        for f in self._paths["l2a"].glob("*L2A*.SAFE"):
            # Nb jp2 < 7
            if len(list(f.glob("GRANULE/*/IMG_DATA/R10m/*.jp2"))) < 7:
                logger.info("Corrupted L2A {} in L2A folder (nb jp2 <7 in R10m folder)".format(f.name))
                nb_id += 1
                if remove:
                    try:
                        shutil.rmtree(str(f))
                        logger.info("Removing corrupted L2A {} from L2A folder".format(f.name))
                        nb_rm += 1
                    except:
                        logger.error("Can't remove {} from L2A folder".format(f.name))
        # identify 0B cloud masks
        for f in self._paths["l2a"].glob("*L2A*_CLOUD_MASK.jp2"):
            if f.stat().st_size == 0:
                logger.info("Corrupted cloud mask {} in L2A folder".format(f.name))
                nb_id += 1
                if remove:
                    logger.info("Removing corrupted cloud mask {} from L2A folder".format(f.name))
                    f.unlink()
                    nb_rm += 1
        # identify wrong size l2a_QL
        for f in self._paths["l2a"].glob("*L2A*_QL.tif"):
            if f.stat().st_size != 3617212:
                logger.info("Corrupted L2A QL {} in L2A folder".format(f.name))
                nb_id += 1
                if remove:
                    logger.info("Removing corrupted QL {} from L2A folder".format(f.name))
                    f.unlink()
                    nb_rm += 1
        # identify 0B or absent indices QL
        for f in self._paths["indices"]:
            #~ logger.info(f, self._paths["indices"][f])
            for p in self._paths["indices"][f].glob("*_MSIL2A_*/"):
                #~ logger.info(p)
                if p.is_file():
                    logger.info("Identified old indice format {}".format(p.name))
                    nb_id += 1
                    if remove:
                        logger.info("Removing old indice format {}".format(p.name))
                        p.unlink()
                        nb_rm += 1
                else:
                    #~ for q in p.glob("*_QUICKLOOK.tif"):
                        #~ if not ((q.stat().st_size == 3617212) or 
                                #~ (q.stat().st_size == 4196652) or
                                #~ (q.stat().st_size == 3617478)):
                            #~ logger.info("Corrupted indice QL {} (bad size)".format(q.name))
                            #~ if remove:
                                #~ logger.info("Removing indice QL {}".format(q.name))
                                #~ q.unlink()
                    for q in list(p.glob("*.jp2")) + list(p.glob("*.tif")):
                        #~ logger.info(q)
                        try:
                            Image.MAX_IMAGE_PIXELS = 120560400
                            img = Image.open(str(q)) # open the image file
                            img.verify() # verify that it is, in fact an image
                        except (IOError, SyntaxError) as e:
                            logger.info('Bad file (PIL): {}'.format(str(q.name))) # print out the names of corrupt files
                            nb_id += 1
                            if remove:
                                logger.info("Removing indice QL {}".format(q.name))
                                q.unlink()
                                nb_rm += 1
                    for q in list(p.glob("*.jp2")):
                        if not (Path(str(q) + ".aux.xml")).exists(): 
                            logger.info("Missing metadata: {}".format(q.name))
                            nb_id += 1
                            if remove:
                                logger.info("Removing jp2 {}".format(q.name))
                                q.unlink()
                                nb_rm += 1
                    for q in p.glob("*.*"):
                        if q.stat().st_size == 0:
                            logger.info("Corrupted file {} (0B size)".format(q.name))
                            nb_id += 1
                            if remove:
                                logger.info("Removing indice QL {}".format(q.name))
                                q.unlink()
                                nb_rm += 1
                    if remove_indice_tif:
                        for q in p.glob("*" + f.upper() + ".tif"):
                            logger.info("Identified indice in tif format {}".format(q.name))
                            nb_id += 1
                            if remove:
                                logger.info("Removing indice QL {}".format(q.name))
                                q.unlink()
                                nb_rm += 1
        return {"identified_problems": nb_id, "removed_problems": nb_rm}
            
    def archive_l1c(self, 
                    size_only: bool = False,):
        """
        Check and move l1c products to archive folder
        
        """
        
        l1c_archive_path = Path(Config().get("l1c_archive_path"))
        
        prod_list = ProductsList()
        archive_l1c_set = {a for a in {identifier.replace("L2A_", "L1C_").replace("_USER_", "__OPER__")
                                                     for identifier in self.l2a.products} if a in set(self.l1c.products)}
        
        for prod in archive_l1c_set:
            prod_list[prod] = {"date": self._products["l1c"][prod].date,
                                "cloud_cover": self._products["l1c"][prod].cloud_cover}
        
        if prod_list:
            count = 0
            total_size = 0 
            for prod in prod_list:
                l1c = L1cProduct(prod.identifier)
                if not l1c.path.is_symlink():
                    count += 1
                    total_size += getFolderSize(str(l1c.path))
                    if not size_only:
                        move_path = l1c_archive_path / l1c.tile / l1c.path.name
                        logger.info("archiving {}".format(l1c.identifier))
                        move_path.parent.mkdir(exist_ok=True)
                        #~ shutil.move(str(l1c.path), str(move_path.parent))
                        distutils.dir_util.copy_tree(str(l1c.path), str(move_path))
                        distutils.dir_util.remove_tree(str(l1c.path))
                        l1c.path.symlink_to(move_path, target_is_directory = True)
            if size_only:
                logger.info("{} l1c product(s) to archive ({})".format(count, human_size(total_size)))
            else:
                logger.info("{} l1c product(s) archived ({})".format(count, human_size(total_size)))
            return total_size
        if not count:
            logger.info("No L1C products to archive")
            return 0
                
    def archive_l2a(self,
                    size_only: bool = False,):
        """
        Check errors and move l2a products to archive folder
        
        """
        
        if (self.clean_lib()['identified_problems']-self.clean_lib()['removed_problems']) == 0:
        
            l2a_archive_path = Path(Config().get("l2a_archive_path"))
            
            prod_list = self.l2a
                                   
            if prod_list:
                count = 0
                total_size = 0 
                for prod in prod_list:
                    l2a = L2aProduct(prod.identifier)
                    if not l2a.path.is_symlink():
                        count += 1
                        total_size += getFolderSize(str(l2a.path))
                        if not size_only:
                            move_path = l2a_archive_path / l2a.tile / l2a.path.name
                            logger.info("archiving {}".format(l2a.identifier))
                            move_path.parent.mkdir(exist_ok=True)
                            shutil.move(str(l2a.path), str(move_path.parent))
                            l2a.path.symlink_to(move_path, target_is_directory = True)
                if size_only:
                    logger.info("{} l2a product(s) to archive ({})".format(count, human_size(total_size)))
                else:
                    logger.info("{} l2a product(s) archived ({})".format(count, human_size(total_size)))
                return total_size
            else:
                logger.info("No L2A products, nothing to archive")
                return 0
        else:
            logger.info("Error(s) in l2a product(s) please correct them running clean_lib(remove=True) before archiving")
            return 0
    
    def archive_all(self, 
                    size_only: bool = False,):
        l1c_size = self.archive_l1c(size_only = size_only)
        l2a_size = self.archive_l2a(size_only = size_only)
        return l1c_size + l2a_size   
    
    def update_latest_ql(self):
        """
        Produce or update the latest l2a quicklook for the tile
        """
        p = self.l2a.last.identifier
        l2a = L2aProduct(p)

        outfullpath = l2a.path.parent / (p[39:44] + '_' + p[0:4] + p[11:19] + '_QL_latest.jpg')
        old_ql = list(l2a.path.parent.glob('*_QL_latest.jpg*'))
        liste=[a for a in old_ql if str(outfullpath) not in str(a)]
        for f in liste:
            f.unlink()
        if outfullpath in old_ql:
            logger.info("{} - Latest QL already done".format(self.name))
            return
        else:
            l2a.process_ql(out_path = outfullpath, out_resolution=(750,750), jpg = True)
        
